% read data
data=xlsread('082420_pure_pellet_PARSEC_data');
v=data(:,1);
% change the number of column to the sample need to run
em=data(:,2);

% cut out data with wavenumbers < 350 cm-1.
em=em(find(v>350)); 
v=v(find(v>350)); 

% and wavenumbers >1600 cm-1
em=em(find(v<1600)); 
v=v(find(v<1600)); 

% Convert emissivity to reflectance
R=1.-em;

% Plot spectra
figure(1)
plot(v,R,'k','LineWidth',3.0);
set(gca,'XDir','reverse','FontSize',10);
ylabel('Reflectance','FontSize',12);
xlabel('Wavenumber (cm^{-1})','FontSize',12);

% Read parameter input file
param_file=importdata('oscillation_params_new.txt');
param_values=param_file.data;

nu=param_values(:,1)';
gamm=param_values(:,2)';
fourpr=param_values(:,3)';
epsil=param_values(1,4);

% gamm=gamm./nu;
% fourpr=Sk./(nu.^2);

% Set lower and upper bounds
lb_nu=-20.+nu;
ub_nu=20.+nu;
lb_gamm=zeros(1,length(nu));
ub_gamm=repmat(2,1,length(nu));
lb_fourpr=zeros(1,length(nu));
ub_fourpr=repmat(2,1,length(nu));

lb=[lb_nu,lb_gamm,lb_fourpr,1];
ub=[ub_nu,ub_gamm,ub_fourpr,4];

% concatenate together guesses to make coefficient vector
coefg=[nu,gamm,fourpr,epsil];

% do nonlinear inversion
options=optimset('Display','iter','MaxIter',10000);
[coef,resnorm,res,exitflag,output,lambda,J]=lsqcurvefit(@disp_model_wrap_sbu,coefg,v,R,lb,ub,options);
%fprintf('%f\n',coefg)
fprintf('%f\n',coef)

% run forward model to produce guess and fitted spectra R_g and R_f
R_g=disp_model_wrap_sbu(coefg,v);
[R_f,n,k]=disp_model_wrap_sbu(coef,v);

% save optical constants 
file=fopen('optical_constants_Fo80.txt','w'); 
fprintf(file,'%s\t%s\t%s\n','wavenumber','n','k');
fclose(file);
dlmwrite('optical_constants_Fo80.txt',cat(2,v,n,k),'-append','delimiter','\t');

% save oscillation parameters
numb=(length(coef)-1)/3;
fnu=coef(1:numb);
fgamm=coef(numb+1:numb*2);
ffourpr=coef(numb*2+1:numb*3);
fepsil=repmat(coef(end),1,numb);
param=cat(2,fnu',fgamm',ffourpr',fepsil');
out=sortrows(param,1);
fid = fopen('oscillation_params_new.txt','w'); 
fprintf(fid,'%s\t\t%s\t\t%s\t\t%s\n','nu','gamm','fourpr','epsil');
fclose(fid);
dlmwrite('oscillation_params_new.txt',out,'-append','precision','%.6f','delimiter','\t');

figure(2); clf;
hold on;
plot(v,R,'k',v,R_f,'r','LineWidth',3.0);
set(gca,'XDir','reverse','FontSize',14)
ylabel('Reflectance','FontSize',14);
xlabel('Wavenumber (cm^{-1})','FontSize',14);
hold off;

figure(3); clf;
plot(v,R-R_f,'k','LineWidth',3.0);
set(gca,'XDir','reverse','FontSize',14)
ylabel('Residual','FontSize',14);
xlabel('Wavenumber (cm^{-1})','FontSize',14);

figure(4); clf;
plot(v,n,'k','LineWidth',3.0);
set(gca,'XDir','reverse','FontSize',14)
ylabel('n','FontSize',14);
xlabel('Wavenumber (cm^{-1})','FontSize',14);

figure(5); clf;
semilogy(v,k,'k','LineWidth',3.0);
set(gca,'XDir','reverse','FontSize',14)
ylabel('k','FontSize',14);
xlabel('Wavenumber (cm^{-1})','FontSize',14);
